# AppStateArgs.pm - The Object Class that provides a AppStateArgs Base Object
# Created by James A. Pattie, 03/26/2003.

# Copyright (c) 2000-2003 Xperience, Inc. http://www.pcxperience.com/
# All rights reserved.  This program is free software; you can redistribute it
# and/or modify it under the same terms as Perl itself.

package Portal::AppState;
use Portal::Base;
use Portal::Data::Variables;
use strict;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '0.04';

=head1 NAME

AppStateArgs - Object used to build a base Application State Object Class.

=head1 SYNOPSIS

 package Portal::Test::Main;
 use Portal::AppState;
 use strict;
 use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

 require Exporter;

 @ISA = qw(Portal::AppState Exporter AutoLoader);
 @EXPORT = qw();

 $VERSION = '0.01';

 my %commands = ( display => "display the state", close => "Close the App", );

 sub new
 {
   my $class = shift;
   my $self = $class->SUPER::new(@_);
   my %args = ( @_ );

   if ($self->error)
   {
     $self->prefixError();
     return $self;
   }

   # instantiate anything unique to this module
   $self->{commands} = \%commands;
   $self->{state} = "Main";  # this states name.

   # do validation
   # The $self->Portal::Test::Main::isValid makes sure we access our
   # isValid method and not any classes isValid method that has
   # derived from us.
   if (!$self->Portal::Test::Main::isValid)
   {
     # the error is set in the isValid() method.
     return $self;
   }

   # do anything else you might need to do.
   return $self;
 }

 =item % getCommands(void)

   This method returns the commands hash and is only to be
   used by the registerApp method for creating the permissions,
   if using the AppStateCommandSecurity implementation.

 =cut
 sub getCommands
 {
   return %commands;
 }

 sub isValid
 {
   my $self = shift;

   # make sure our Parent class is valid.
   if (!$self->SUPER::isValid())
   {
     $self->prefixError();
     return 0;
   }

   # validate our parameters.

   if ($self->numInvalid() > 0 || $self->numMissing() > 0)
   {
     $self->error($self->genErrorString("all"));
     return 0;
   }
   return 1;
 }

=head1 DESCRIPTION

AppStateArgs is the base class for Portal Application State modules.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(langObj, configObj, cookieObj, input, dbHandle,
                 browserType, browserVersion, browserMaker,
                 methods, appObj, applicationObj, userObj,
                 companyObj, portalSession, authObj, logObj,
                 sessionObj, appConfigObj, app, appVersion,
                 logOk, useAppStateCommandSecurity, states)

Creates a new instance of the Portal::AppState
object.

requires: langObj - Portal::Language Object
          configObj - Portal::Data::Config
          appConfigObj - Portal::Data::Config
                         config object for the App in question
          cookieObj - hash of cookies
          input - hash of passed in arguments
          dbHandle - DBIWrapper pointing at the App's
                     Database
          browserType
          browserVersion
          browserMaker
          methods - Portal::Methods Object
          appObj - Current applications info from the
                   Portal Database
          applicationObj - Portal::Application Object
          userObj - Portal::Objects::User
          companyObj - Portal::Objects::CompanyObject
          portalSession - Portal::Session Object
          sessionObj - Portal::Session Object for the
                       Application
          logObj - Portal::Log Object
          authObj - Portal::Auth Object

          The following are provided by the derived
          Portal::App instance of your application.

          app   - Name of the Portal App
          appVersion - Version of your Portal App
          logOk - Do we have access to a log_tb?
          useAppStateCommandSecurity
          states - hash of states for the App.


          Provides:
          portalVariables - an instance of the
            Portal::Data::Variables module.

returns:  object reference

The langObj requires the following phrases:

B<missingArgument> - "%s is missing"

B<invalidArgument> - "%s = '%s' is invalid"

B<errorExclam> - "Error!"

=cut

sub new
{
  my $class = shift;
  my %args = ( configObj => undef, cookieObj => undef, input => undef, dbHandle => undef,
               browserType => "HTML", browserVersion => "4.0", appConfigObj => undef,
               browserMaker => "", methods => undef, appObj => undef,
               applicationObj => undef, userObj => undef, companyObj => undef,
               portalSession => undef, authObj => undef, logObj => undef, sessionObj => undef,
               app => "", appVersion => "", logOk => 0, states => undef,
               useAppStateCommandSecurity => 0, @_ );
  my $self = $class->SUPER::new(@_);

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{applicationObj} = $args{applicationObj};  # The Portal::Application module.
  $self->{input} = $args{'input'};
  $self->{cookieObj} = $args{cookieObj};
  $self->{configObj} = $args{configObj};
  $self->{appConfigObj} = $args{appConfigObj};
  $self->{appObj} = $args{appObj};  # info about the application being run.
  $self->{userObj} = $args{userObj};
  $self->{companyObj} = $args{companyObj};
  $self->{portalSession} = $args{portalSession};
  $self->{browserType} = $args{browserType};
  $self->{browserVersion} = $args{browserVersion};
  $self->{browserMaker} = $args{browserMaker};
  $self->{authObj} = $args{authObj};
  $self->{methods} = $args{methods};
  $self->{dbHandle} = $args{dbHandle};
  $self->{sessionObj} = $args{sessionObj};
  $self->{logObj} = $args{logObj};
  $self->{app} = $args{app};
  $self->{appVersion} = $args{appVersion};
  $self->{logOk} = $args{logOk};
  $self->{states} = $args{states};
  $self->{useAppStateCommandSecurity} = $args{useAppStateCommandSecurity};

  # variables unique to the State.
  $self->{commands} = {};

  # do validation
  if (!$self->Portal::AppState::isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.
  $self->{portalVariables} = Portal::Data::Variables->new(langObj => $self->{langObj});
  if ($self->{portalVariables}->error)
  {
    $self->error($self->{portalVariables}->errorMessage);
    return $self;
  }

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.
  if (not defined $self->{input})
  {
    $self->missing("input");
  }
  if (not defined $self->{portalSession})
  {
    $self->missing("portalSession");
  }
  elsif (!$self->{portalSession}->isValid)
  {
    $self->invalid("portalSession", $self->{portalSession}->errorMessage);
  }
  if (not defined $self->{sessionObj})
  {
    $self->missing("sessionObj");
  }
  elsif (!$self->{sessionObj}->isValid)
  {
    $self->invalid("sessionObj", $self->{sessionObj}->errorMessage);
  }
  if (not defined $self->{appObj})
  {
    $self->missing("appObj");
  }
  elsif (!$self->{appObj}->isValid)
  {
    $self->error($self->{appObj}->errorMessage);
  }
  if (not defined $self->{applicationObj})
  {
    $self->missing("applicationObj");
  }
  elsif (!$self->{applicationObj}->isValid)
  {
    $self->error($self->{applicationObj}->errorMessage);
  }
  if (not defined $self->{userObj})
  {
    $self->missing("userObj");
  }
  elsif (!$self->{userObj}->isValid)
  {
    $self->error($self->{userObj}->errorMessage);
  }
  if (not defined $self->{companyObj})
  {
    $self->missing("companyObj");
  }
  elsif (!$self->{companyObj}->isValid)
  {
    $self->error($self->{companyObj}->errorMessage);
  }
  if (not defined $self->{authObj})
  {
    $self->missing("authObj");
  }
  elsif (!$self->{authObj}->isValid)
  {
    $self->error($self->{authObj}->errorMessage);
  }
  if (not defined $self->{cookieObj})
  {
    $self->missing("cookieObj");
  }
  if (not defined $self->{configObj})
  {
    $self->missing("configObj");
  }
  if (not defined $self->{appConfigObj})
  {
    $self->missing("appConfigObj");
  }
  if ($self->{browserType} !~ /^(HTML|HDML|WML)$/)
  {
    $self->invalid("browserType", $self->{browserType});
  }
  if (length $self->{browserVersion} == 0)
  {
    $self->invalid("browserVersion", $self->{browserVersion});
  }
  if (not defined $self->{methods})
  {
    $self->missing("methods");
  }
  elsif (!$self->{methods}->isValid)
  {
    $self->error($self->{methods}->errorMessage);
  }
  if (not defined $self->{dbHandle})
  {
    $self->missing("dbHandle");
  }
  if (not defined $self->{logObj})
  {
    $self->missing("logObj");
  }
  elsif (!$self->{logObj}->isValid)
  {
    $self->error($self->{logObj}->errorMessage);
  }
  if ($self->{app} !~ /^(.+)$/)
  {
    $self->invalid("app", $self->{app});
  }
  if ($self->{appVersion} !~ /^(.+)$/)
  {
    $self->invalid("appVersion", $self->{appVersion});
  }
  if (! defined $self->{states})
  {
    $self->missing("states");
  }
  if ($self->{useAppStateCommandSecurity} !~ /^(0|1)$/)
  {
    $self->invalid("useAppStateCommandSecurity", $self->{useAppStateCommandSecurity});
  }
  if ($self->{logOk} !~ /^(0|1)$/)
  {
    $self->invalid("logOk", $self->{logOk});
  }

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }
  if ($self->error)
  {
    return 0;
  }

  return 1;
}

=item HTMLObject run(command)

 takes: command
 returns: The HTMLObject document to display or undef if an error.

 summary: We validate the specified command against the
   $self->{commands} hash.  If the command exists, we then
   sanitize it (replace any non a-z, A-Z, 0-9 or _ character with
   an _) and execute the function that is named c_{command} where
   {command} is the sanitized version.  This is in the scope of
   $self, so it has to be defined as a method of the derived
   Portal::AppState module.

   Ex:  State = Portal::Example::Main, command = display
   would exectue $self->c_display();

   c_display needs to be defined as follows:

   sub c_display
   {
     my $self = shift;

     my $doc = $self->setupCSS();
     if ($self->error)
     {
       $self->prefixError();
       return $doc;
     }

     # do the display code here

     return $doc;
   }

=cut
sub run
{
  my $self = shift;
  my %args = ( command => "", @_ );
  my $command = $args{command};
  my $doc;

  if (!exists $self->{commands}->{$command})
  {
    $self->error(sprintf($self->langObj->map("commandNotKnown"), $command));
    return undef;
  }

  (my $safeCommand = $command) =~ s/([^\w])/_/g;

  # now we execute the c_ command function
  eval "\$doc = \$self->c_" . $safeCommand . "();";
  if ($@)
  {
    $self->prefixError($@);
    return undef;
  }

  return $doc;
}

=item HTMLObject setupCSS(doc)

  requires:
  optional: doc
  returns:  doc

  summary:  This method will instantiate an HTMLObject::Normal
            document, if you don't specify doc, and then proceeds
            to get the current color scheme for the user and
            create the CSS entries in the doc.  We also call
            Portal::Methods->includeCoreJSFiles() to pull in
            any JavaScript files that the Portal requires for
            the generatePickerCode() method to "just work".

=cut
sub setupCSS
{
  my $self = shift;
  my %args = ( doc => undef, @_ );
  my $doc = $args{doc};
  my $applicationObj = $self->{applicationObj};
  my $userId = $self->{userObj}->{id};

  if (!defined $doc)
  {
    $doc = HTMLObject::Normal->new;
  }

  # get the users colorScheme to work with.
  my $colorScheme = $applicationObj->getCurrentColorScheme(userId => $userId);
  if ($applicationObj->error)
  {
    $self->error($applicationObj->errorMessage);
    return $doc;
  }
  if (!defined $colorScheme)
  {
    $self->error($self->{langObj}->map("noValidColorScheme"));
    return $doc;
  }

  # create the needed css entries
  my %cssEntries = $self->{methods}->generateColorSchemeCSS(colorScheme => $colorScheme, appName => $self->{app});
  if ($self->{methods}->error)
  {
    $self->error($self->{methods}->errorMessage);
    return $doc;
  }
  $doc->print(%cssEntries);

  # make sure that this doc supports javascript.
  if (exists $doc->{javascriptIncludes})
  {
    $self->{methods}->includeCoreJSFiles(doc => $doc);
    if ($self->{methods}->error)
    {
      $self->error($self->{methods}->errorMessage);
      return $doc;
    }
  }

  return $doc;
}

=item HTMLObject prepDoc(doc, docType, title, titlePhrase, template, templateStr)

 doc - an HTMLObject instance to work with instead of instantiating
   a new HTMLObject document.  Defaults to undef, so a new doc
   will be generated by default.

 docType - specify the type of HTMLObject document to be created.
   Valid values are: 'Base', 'Normal', 'FrameSet'
   Default is 'Normal'.

   If 'Base' is specified, then the core javascript modules will
   not be included, but CSS will always be setup.

 title - specify the actual title string to be set in the Document.

 titlePhrase - specify the phrase to be looked up in the langObj and
   used for the title.

 If neither title or titlePhrase are specified, then the title will
   not be set.

 template - specify the type of template html code to be generated in
   the body of the document.  Valid values are: 'none', 'title',
   'menubar', 'menubar+sidemenu-left', 'menubar+sidemenu-right' or
   'custom'.  Defaults to 'none'.

   If 'none', then nothing will be output into the body of the document.

   If 'title', then

   <center><h3>$title</h3></center>

   will be output into the body of the document, where $title is the
   title or titlePhrase you specified.  If you did not specify a title
   or titlePhrase value, then <title> will be output instead allowing
   you to substitute later.

   If 'menubar', then

   <menubar>
   <br />

   will be output into the body of the document so you can later
   substitute in the generated menubar.

   If 'menubar+sidemenu-left', then

   <menubar>
   <br />
   <table border="0" cellspacing="0" cellpadding="2" width="100%">
     <tr>
       <td width="25%" valign="top"><sidemenu></td>
       <td class="main" width="75%" align="center"><maincontent></td>
     </tr>
   </table>

   will be output into the body of the document so you can later
   substitute <menubar> for the main menu, <sidemenu> for the side
   menu and <maincontent> for the content being displayed in the main
   table.

   If 'menubar+sidemenu-right', then

   <menubar>
   <br />
   <table border="0" cellspacing="0" cellpadding="2" width="100%">
     <tr>
       <td class="main" width="75%" align="center"><maincontent></td>
       <td width="25%" valign="top"><sidemenu></td>
     </tr>
   </table>

   will be output into the body of the document so you can later
   substitute <menubar> for the main menu, <sidemenu> for the side
   menu and <maincontent> for the content being displayed in the main
   table.

   If 'custom', then the templateStr value will be used to allow for
   custom body html to be generated.

=cut
sub prepDoc
{
}

=item HTMLObject processCloseAppEvent(doc, langEntry, log)

  requires: log (bool)
  optional: doc, langEntry
  returns:  doc

  summary:  This method does all the cleanup work associated with
            closing an app.  This entails closing all registered
            windows, the apps help window, if it is open, and
            then deleting the apps session.

            If your langObj doesn't have a "close" entry, then
            use langEntry to specify the string to be displayed
            that indicates you are closing the app.

  Your langObj has to have the "title" entry which takes 2 parameters
  the version and the current location of the program.

  If you don't pass in doc, then we instantiate an instance of the
  HTMLObject::Normal to work with.

=cut
sub processCloseAppEvent
{
  my $self = shift;
  my %args = ( doc => undef, log => 0, langEntry => "close", @_ );
  my $doc = $args{doc};
  my $langObj = $self->{langObj};
  my $portalSession = $self->{portalSession};
  my $sessionObj = $self->{sessionObj};
  my $applicationObj = $self->{applicationObj};
  my $userId = $self->{userObj}->{id};
  my $log = $args{log};
  my $logObj = $self->{logObj};
  my $langEntry = $args{langEntry};

  if ($log !~ /^(0|1)$/)
  {
    $self->invalid("log", $log);
  }
  if ($log && !defined $logObj)
  {
    $self->missing("logObj");
  }
  if (defined $logObj && !$logObj->isValid)
  {
    $self->error($logObj->errorMessage);
  }
  if ($langEntry !~ /^(.+)$/)
  {
    $self->invalid("langEntry", $langEntry, $self->{langObj}->map("langEntryNeeded"));
  }
  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return $doc;
  }
  if ($self->error)
  {
    return $doc;
  }

  if (!$sessionObj->{store}->{okToLogout})
  {
    $doc = $self->run(command => "display");
    if ($self->error)
    {
      $self->prefixError();
      return $doc;
    }
  }
  else
  {
    $doc = $self->setupCSS(doc => $doc);
    if ($self->error)
    {
      $self->prefixError();
      return $doc;
    }

    my $close = $langObj->map($langEntry);
    my $title = sprintf($langObj->map("title"), $self->{appVersion}, $close);

    # generate the document that will close all open windows and then close itself.
    $doc->setTitle($title);

    # create the closeWindow JS function.
    $doc = $self->{methods}->displayJSHelper(doc => $doc, type => "closeWindow", langObj => $langObj);
    if ($self->{methods}->error)
    {
      $self->error($self->{methods}->errorMessage);
      return $doc;
    }

    my $onLoadCode = "";
    foreach my $window ($sessionObj->getWindows())
    {
      $onLoadCode .= "closeWindow('$window');\n";
    }

    my $windowName = $portalSession->generateHelpWindowName($self->{app});
    if ($portalSession->error)
    {
      $self->error($portalSession->errorMessage);
      return $doc;
    }
    if ($portalSession->windowExists($windowName))
    {
      $onLoadCode .= "closeWindow('$windowName');\n";
      $portalSession->unregisterWindow($windowName);
      if ($portalSession->error)
      {
        $self->error($portalSession->errorMessage);
        return $doc;
      }
    }

    $onLoadCode .= "\nwindow.close();\n";  # close ourself

    if (length $onLoadCode > 0)
    {
      $doc->setOnload(code => "$onLoadCode");  # output the JavaScript code that will be run on the onLoad event of the page.
    }

    $sessionObj->delete;
    if ($sessionObj->error)
    {
      $self->error($sessionObj->errorMessage);
      return $doc;
    }

    if ($log)
    {
      $self->doLog(action => 19, extraInfo => $self->{app}, userId => $userId, logObj => $logObj);
      if ($self->error)
      {
        $self->prefixError();
        return $doc;
      }
    }
  }

  return $doc;
}

=item scalar arguments(void)

 Returns a string that when eval'ed will specify all the
 arguments to pass into an instance of the Portal::AppState->new
 method.

 Ex:

 "langObj => \$self->{langObj}, input => \$self->{input}"

=cut

sub arguments
{
  my $self = shift;
  my $args = $self->SUPER::arguments();

  foreach my $arg (qw(cookieObj input logObj configObj appConfigObj methods authObj applicationObj browserType browserVersion browserMaker dbHandle appObj userObj companyObj portalSession sessionObj app appVersion states commands logOk useAppStateCommandSecurity portalVariables))
  {
    $args .= ", " if ($args);
    $args .= "$arg => \$self->{$arg}";
  }

  return $args;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

James A. Pattie (mailto:james@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Base(3)

=cut
